@@ -17,6 +17,7 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 =end
 
+require 'zlib'
 require 'net/http'
 # get openssl if available
 begin
@@ -33,7 +34,14 @@ HTTPDEBUG = false
 
 # Class used to retrieve the feed over HTTP
 class HTTPFetcher
-  def HTTPFetcher::fetcher(baseuri, uri, lastcheck, recursion)
+
+  @timeout = 30 # should be enough for everybody...
+
+  def timeout=(value)
+    @timeout = value
+  end
+
+  def fetcher(baseuri, uri, lastcheck, recursion)
     proxy_host = nil
     proxy_port = nil
     proxy_user = nil
@@ -49,8 +57,8 @@ class HTTPFetcher
                             proxy_port,
                             proxy_user,
                             proxy_pass ).new(uri.host, uri.port)
-    http.read_timeout = 30 # should be enough for everybody...
-    http.open_timeout = 30
+    http.read_timeout = @timeout
+    http.open_timeout = @timeout
     if uri.scheme == 'https'
       http.use_ssl = true 
       http.verify_mode = OpenSSL::SSL::VERIFY_NONE
@@ -61,11 +69,14 @@ class HTTPFetcher
       useragent = 'Feed2Imap http://home.gna.org/feed2imap/'
     end
 
-    if lastcheck == Time::at(0)
-      req = Net::HTTP::Get::new(uri.request_uri, {'User-Agent' => useragent })
-    else
-      req = Net::HTTP::Get::new(uri.request_uri, {'User-Agent' => useragent, 'If-Modified-Since' => lastcheck.httpdate})
+    headers = {
+      'User-Agent' => useragent,
+      'Accept-Encoding' => 'gzip',
+    }
+    if lastcheck != Time::at(0)
+      headers.merge!('If-Modified-Since' => lastcheck.httpdate)
     end
+    req = Net::HTTP::Get::new(uri.request_uri, headers)
     if uri.userinfo
       login, pw = uri.userinfo.split(':')
       req.basic_auth(login, pw)
@@ -81,7 +92,12 @@ class HTTPFetcher
     end
     case response
     when Net::HTTPSuccess
-      return response.body
+      case response['Content-Encoding']
+      when 'gzip'
+        return Zlib::GzipReader.new(StringIO.new(response.body)).read
+      else
+        return response.body
+      end
     when Net::HTTPRedirection
       # if not modified
       if Net::HTTPNotModified === response
@@ -99,8 +115,8 @@ class HTTPFetcher
     end
   end
     
-  def HTTPFetcher::fetch(url, lastcheck)
+  def fetch(url, lastcheck)
     uri = URI::parse(url)
-    return HTTPFetcher::fetcher(uri, uri, lastcheck, MAXREDIR)
+    return fetcher(uri, uri, lastcheck, MAXREDIR)
   end
 end
